@@ -8,7 +8,8 @@ from azureml.core import (
     Workspace,
 )
 from azureml.core.compute import AmlCompute, ComputeTarget
-from azureml.core.runconfig import MpiConfiguration
+from azureml.core.runconfig import MpiConfiguration, DockerConfiguration
+from azureml.core.authentication import ServicePrincipalAuthentication
 from azureml.tensorboard import Tensorboard
 from tabulate import tabulate
 
@@ -24,27 +25,26 @@ available_instances = {
     "2xV100": "Standard_NC12s_v3",
     "4xV100": "Standard_NC24s_v3",
     "8xV100": "Standard_ND40rs_v2",
-    "CPU": "Standard_D8_v3",
+    "CPU": "STANDARD_DS11_V2",  # "Standard_D8_v3",
 }
 
 
-def get_environment(requirements_file: Path, docker_image: str):
+def get_environment(workspace, environment_name: str, requirements_file: Path):
+
     if not requirements_file.exists():
         raise RuntimeError(f"Given requirements file '{requirements_file}' does not exist at the provided path")
     elif requirements_file.name.endswith(".txt"):
-        env = Environment.from_pip_requirements("my-pip-env", requirements_file)
+        env = Environment.from_pip_requirements(environment_name, requirements_file)
     elif requirements_file.name.endswith(".yml"):
-        env = Environment.from_conda_specification("my-env", requirements_file)
+        env = Environment.from_conda_specification(environment_name, requirements_file)
     else:
         print("Couldn't resolve env from requirements file")
 
-    env.docker.enabled = True
-    env.docker.base_image = docker_image
     return env
 
 
 def get_inferred_instance(gpu_type: str, num_gpus: int):
-    if gpu_type is None:
+    if gpu_type is None or gpu_type == "CPU":
         return available_instances.get("CPU")
     instance = available_instances.get(f"{num_gpus}x{gpu_type}")
     assert instance is not None, f"Could not look up inferred instance w/ Num GPUs: {num_gpus} and GPU Type: {gpu_type}"
@@ -88,15 +88,48 @@ def submit_basic_run(
     num_nodes=1,
     dataset_name=None,
     dataset_mount_dir="/dataset",
-    experiment_name="pycloud-dev",
-    requirements_file="./environment.yml",
-    docker_image="mcr.microsoft.com/azureml/openmpi3.1.2-cuda10.2-cudnn8-ubuntu18.04",
+    experiment_name="fuego-experiments",
+    environment_name='my-env',
+    requirements_file="./requirements.txt",
+    docker_image="mcr.microsoft.com/azureml/openmpi4.1.0-cuda11.3-cudnn8-ubuntu20.04",
+    subscription_id=None,
+    resource_group=None,
+    workspace_name=None,
+    tenant_id=None,
+    client_id=None,
+    client_secret=None,
 ):
-    ws = Workspace.from_config(workspace_config)
+    do_use_service_principal = all([tenant_id, client_id, client_secret])
+    if do_use_service_principal:
+        auth = ServicePrincipalAuthentication(
+            tenant_id=tenant_id,
+            service_principal_id=client_id,
+            service_principal_password=client_secret,
+        )
+    else:
+        auth = None
+
+
+    do_use_config_file = not all([subscription_id, resource_group, workspace_name])
+    if do_use_config_file:
+        ws = Workspace.from_config(workspace_config, auth=auth)
+    else:
+        ws = Workspace(
+            subscription_id=subscription_id,
+            resource_group=resource_group,
+            workspace_name=workspace_name,
+            auth=auth
+        )
+
     compute_target = find_or_create_compute_target(
         ws, target_name, get_inferred_instance(gpu_type, num_gpus), min_nodes, max_nodes
     )
-    env = get_environment(requirements_file, docker_image)
+
+    # Get/Create the environment + specify docker image/configuration.
+    env = get_environment(ws, environment_name, requirements_file)
+    docker_config = DockerConfiguration(use_docker=True)
+    env.docker.base_image = docker_image
+
     run_config = ScriptRunConfig(
         source_directory=Path(script).parent,
         script=Path(script).name,
@@ -104,6 +137,7 @@ def submit_basic_run(
         compute_target=compute_target,
         environment=env,
         distributed_job_config=MpiConfiguration(process_count_per_node=1, node_count=num_nodes),
+        docker_runtime_config=docker_config,
     )
     if dataset_name is not None:
         ds = ws.datasets.get(dataset_name)
@@ -150,5 +184,11 @@ def create_tensorboard(workspace_config, experiment_name):
     exp = Experiment(ws, experiment_name)
     tb = Tensorboard([*exp.get_runs()])
     tb.start()
-    _ = input("press any key to stop tensorboard")
+    print("Check out your Tensorboard at: http://localhost:6006")
+
+    while True:
+        input_data = input("Stop the tensorboard server? [y]")
+        if input_data in ['y']:
+            break
+
     tb.stop()
